#include "timer.h"
#include "vec2.h"
#include "imageptr.h"
#include "range2.h"
#include "transform.h"
#include "system.h"
#include "plot.h"
#include <stdio.h>

void callback_save_pgm(void *params, const struct codeblock_t *codeblock)
{
	UNUSED(params);

#ifdef CONFIG_CODEBLOCK_PARAMETERS
	if( 0 == codeblock->r )
#endif
	{
		// NOTE: codeblock->llband of size cb_size1
		const struct vec2_t c0 = convert_tc_to_next_tb_ll(codeblock->local_c0);
		const struct vec2_t c1 = convert_tc_to_next_tb_ll(codeblock->local_c1);
		const struct vec2_t size = vec2_sub(c1, c0);

		if( size.x > 0 && size.y > 0 )
		{
			imageptr_log_dump_viewport_fmt(
				codeblock->llband,
				c0,
				size,
				"codeblocks/cb_n%i_x%i_y%i_LL_viewport.pgm",
				codeblock->n,
				codeblock->tr.x,
				codeblock->tr.y
			);
		}
	}

	{
		// NOTE: codeblock->hlband of size cb_size1
		const struct vec2_t c0 = convert_tc_to_next_tb_hl(codeblock->local_c0);
		const struct vec2_t c1 = convert_tc_to_next_tb_hl(codeblock->local_c1);
		const struct vec2_t size = vec2_sub(c1, c0);

		if( size.x > 0 && size.y > 0 )
		{
			imageptr_log_dump_viewport_fmt(
				codeblock->hlband,
				c0,
				size,
				"codeblocks/cb_n%i_x%i_y%i_HL_viewport.pgm",
				codeblock->n,
				codeblock->tr.x,
				codeblock->tr.y
			);
		}
	}

	{
		// NOTE: codeblock->lhband of size cb_size1
		const struct vec2_t c0 = convert_tc_to_next_tb_lh(codeblock->local_c0);
		const struct vec2_t c1 = convert_tc_to_next_tb_lh(codeblock->local_c1);
		const struct vec2_t size = vec2_sub(c1, c0);

		if( size.x > 0 && size.y > 0 )
		{
			imageptr_log_dump_viewport_fmt(
				codeblock->lhband,
				c0,
				size,
				"codeblocks/cb_n%i_x%i_y%i_LH_viewport.pgm",
				codeblock->n,
				codeblock->tr.x,
				codeblock->tr.y
			);
		}
	}

	{
		// NOTE: codeblock->hhband of size cb_size1
		const struct vec2_t c0 =  convert_tc_to_next_tb_hh(codeblock->local_c0);
		const struct vec2_t c1 =  convert_tc_to_next_tb_hh(codeblock->local_c1);
		const struct vec2_t size = vec2_sub(c1, c0);

		if( size.x > 0 && size.y > 0 )
		{
			imageptr_log_dump_viewport_fmt(
				codeblock->hhband,
				c0,
				size,
				"codeblocks/cb_n%i_x%i_y%i_HH_viewport.pgm",
				codeblock->n,
				codeblock->tr.x,
				codeblock->tr.y
			);
		}
	}
}

int main()
{
	system_setup_debug();

	dprintf("coordinate format is (x,y)\n");

	const int N = 8;

	// code-blocks of the size of (2^6, 2^6)
	const struct vec2_t cb_exp = { 6, 6 };

	const struct vec2_t tc0 = { 512+5, 512+0 };
	const struct vec2_t size = { 492, 1284 };
	const struct vec2_t tc1 = vec2_add(tc0, size);

	struct imageptr_t *imageptr = imageptr_create_sparse(size);

#if 1
	imageptr_fill(
		imageptr,
		size
	);
#endif

	struct transform_t *transform = transform_create(
		tc0,
		tc1,
		cb_exp,
		1,
		N,
#if 0
		 NULL,
#else
		callback_save_pgm,
#endif
		NULL
	);

	if( !transform )
		return -1;

	while( !transform_finished(transform) )
	{
		struct range2_t lines = transform_prepare_strip(transform);
#if 1
		// pointing to i0-th row of the tile
		const struct vec2_t data_offset = { 0, lines.i0 };
#else
		// pointing to the beginning of the tile
		const struct vec2_t data_offset = vec2_zero;
#endif
		const struct vec2_t data_size = { size.x, lines.i1 - lines.i0 };

		dprintf("REQUIRED: %i lines starting from %i (local coordinates in the input image), i.e. y [%i..%i]\n", data_size.y, data_offset.y, lines.i0, lines.i1 - 1);

		struct imageptr_t *data_lines = imageptr_viewport(imageptr, data_offset);
#if 0
		imageptr_fill_strip(
			data_lines,
			data_offset,
			data_size
		);
#else
		UNUSED(data_size);
#endif
		transform_process_strip(
			transform,
			data_lines,
			data_offset
		);

		free(data_lines);

		// discard already processed data
		imageptr_discard_sparse(
			imageptr,
			vec2_create( 0, lines.i1 )
		);
	}

#if 0
	imageptr_dump(imageptr, size, "image.pgm");
#endif

	transform_dump(transform);

	transform_destroy(transform);

	imageptr_destroy_sparse(imageptr, size);

	return 0;
}
